using System;
using System.IO;
using System.Web.Services.Protocols;

using VaultClientOperationsLib;
using VaultClientNetLib;
using VaultLib;

namespace support
{
	/// <summary>
	/// Summary description for Class1.
	/// </summary>
	class SupportClient
	{
		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main(string[] args)
		{
			// You can change these parameters (or read them from another source).
			string username = "admin";
			string password = "admin";
			string server = "localhost";
			string repositoryName = "Test Repository";

			// Initialize the client instance object.
			ClientInstance ci = new ClientInstance();
			ci.Init(VaultConnection.AccessLevelType.Client);

			// Login to the server.  Use "https://" for SSL.
			ci.Login("http://" + server + "/VaultService", username, password);

			// Find the correct repository by name.
			VaultRepositoryInfo correctRepository = null;
			VaultRepositoryInfo[] reps = null;
			ci.ListRepositories(ref reps);
			foreach (VaultRepositoryInfo info in reps)
			{
				if (String.Compare(info.RepName, repositoryName, true) == 0)
				{
					correctRepository = info;
					break;
				}
			}

			if (correctRepository == null)
			{
				Console.WriteLine("Couldn't find repository '{0}'.", repositoryName);
				return;
			}

			// Set the repository we found as the locally active repository.
			ci.SetActiveRepositoryID(correctRepository.RepID, username, correctRepository.UniqueRepID, true, false);

			// Since most of the diff work done by the Vault clients is done at a high level,
			// it's a manual process to do it from the API.  This example application gets the
			// path to the diff app from the user's Vault settings, but this can be changed to
			// any executable diff app.

			// This is the full repository path of the file we're retrieving.
			string fileRepositoryPath = "$/file.txt";

			// These are the two versions we're diffing.  Vault considers -1 to mean "latest", so
			// you can use that if you want one of these to always be the server's most recent version.
			long firstVersion = 1;
			long secondVersion = 2;

			// We need two temp folders because we'll retrieve two files, which will probably have the
			// same name, and we don't want to overwrite the other.
			string tempFolder1 = Path.Combine(Path.GetTempPath(), Guid.NewGuid().ToString());
			string tempFolder2 = Path.Combine(Path.GetTempPath(), Guid.NewGuid().ToString());

			try
			{
				Directory.CreateDirectory(tempFolder1);
				Directory.CreateDirectory(tempFolder2);

				#region Get both versions to a temporary folder

				// Find the file by path in our current tree cache.
				VaultClientFile foundFile = ci.TreeCache.Repository.Root.FindFileRecursive(fileRepositoryPath);

				if (foundFile == null)
				{
					Console.WriteLine("Couldn't find file {0} in the tree.", foundFile);
					return;
				}

				// This is important: do not modify the VaultClientFile reference returned by FindFileRecursive,
				// as you will modify your tree cache (which will be saved to disk).  Make a copy of the
				// object and modify that one.

				VaultClientFile copy = new VaultClientFile(foundFile);

				// Modify the copy to represent the first version of the file.
				copy.Version = firstVersion;

				// Get the first file.
				ci.GetByDisplayVersionToNonWorkingFolder(copy, MakeWritableType.MakeAllFilesWritable, SetFileTimeType.Current, copy.Parent.FullPath, tempFolder1, null);

				// Modify the copy to represent the other version we want.
				copy.Version = secondVersion;

				// Get the first file.
				ci.GetByDisplayVersionToNonWorkingFolder(copy, MakeWritableType.MakeAllFilesWritable, SetFileTimeType.Current, copy.Parent.FullPath, tempFolder2, null);

				#endregion

				#region Diff the files

				string diffApplication = ci.UserOptions.GetString(VaultOptions.CustomDiff);
				string diffArgsFormatString = ci.UserOptions.GetString(VaultOptions.CustomDiff_Args);

				// Process the arg format string to insert the correct paths and captions.
				string processedArgString = ci.ReplaceDiffArgs(
					diffArgsFormatString, 
					Path.Combine(tempFolder1, copy.Name),
					copy.Name + " " + firstVersion.ToString(),
					Path.Combine(tempFolder2, copy.Name),
					copy.Name + " " + secondVersion.ToString());
				
				Console.WriteLine("Launching diff application.");

				System.Diagnostics.Process diffProc = new System.Diagnostics.Process();
				diffProc.StartInfo.FileName = diffApplication;
				diffProc.StartInfo.Arguments = processedArgString;
				diffProc.Start();

				Console.WriteLine("Waiting for application exit...");
				diffProc.WaitForExit();
				Console.WriteLine("Diff application exited.");

				#endregion
			}
			finally
			{
				// Clean up the temp files.

				if (Directory.Exists(tempFolder1))
					Directory.Delete(tempFolder1, true);

				if (Directory.Exists(tempFolder2))
					Directory.Delete(tempFolder2, true);
			}

			// Remember to logout.
			ci.Logout();

			// Exists so we can debug this project in Visual Studio and see any output.
			Console.WriteLine("Press Enter to finish.");
			Console.ReadLine();
		}
	}
}
